library test_package_demo;

import 'package:flutter/material.dart';

class FriendIndexBar extends StatefulWidget {
  final List<String> listIndexLetter;
  final void Function(int index) indexBarCallback;
  const FriendIndexBar({
    required this.listIndexLetter,
    required this.indexBarCallback,
    Key? key,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _FriendIndexBarState();
}

class _FriendIndexBarState extends State<FriendIndexBar> {
  Color _bgColor = Color.fromRGBO(1, 1, 1, 0);
  Color _textColor = Colors.black;
  List<Widget> _listIndexLetterWidget = [];

  double _indicatorY = -1.1;
  String _indicatorStr = 'A';
  bool _isHide = true;

  @override
  void initState(){
    super.initState();

    for (int i=0; i<widget.listIndexLetter.length; i++) {
      _listIndexLetterWidget.add(
          Expanded(
              child: Container(
                margin: EdgeInsets.all(5),
                child: Text(
                  widget.listIndexLetter[i],
                  style: TextStyle(
                    fontSize: 10,
                    color: _textColor,
                  ),
                ),
              )
          )
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Positioned(
      right: 0,
      top: screenHeight(context)/8,
      width: 70,
      height: screenHeight(context)/2,
      child: Row(
        children: [
          Container(
            alignment: Alignment(0, _indicatorY),
            width: 50,
            child: _isHide ? Container() : Stack(
              alignment: Alignment(-0.2, 0),
              children: [
                Image(
                  image: const AssetImage('images/chat_bubble@3x.png', package: 'test_package_sky_demo'),
                  width: 40,
                ),
                Text(
                  _indicatorStr,
                  style: TextStyle(
                      fontSize: 25,
                      color: Colors.white
                  ),
                )
              ],
            ),
          ),
          GestureDetector(
            onVerticalDragUpdate: (DragUpdateDetails details){
              RenderBox box = context.findRenderObject() as RenderBox;
              Offset offset = box.globalToLocal(details.globalPosition);
              double eachHeight = screenHeight(context)/2 / widget.listIndexLetter.length;
              int index = (offset.dy ~/ eachHeight).clamp(0, widget.listIndexLetter.length-1);
              widget.indexBarCallback(index);
              setState(() {
                _indicatorStr = widget.listIndexLetter[index];
                _indicatorY = (1.1*2) / widget.listIndexLetter.length * index - 1.04;
                _isHide = false;
              });
            },
            onVerticalDragDown: (DragDownDetails details){
              setState(() {
                _bgColor = Color.fromRGBO(1, 1, 1, 0.3);
                _textColor = Colors.white;
              });
            },
            onVerticalDragEnd : (DragEndDetails details){
              setState(() {
                _bgColor = Color.fromRGBO(1, 1, 1, 0);
                _textColor = Colors.black;
                _isHide = true;
              });
            },
            child: Container(
              color: _bgColor,
              width: 20,
              child: Column(
                children: _listIndexLetterWidget,
              ),
            ),
          ),
        ],
      ),
    );
  }
}

//主题颜色
final Color themeColor = Color.fromRGBO(220, 220, 220, 1.0);

//屏幕宽高
double screenWidth(BuildContext context) => MediaQuery.of(context).size.width;
double screenHeight(BuildContext context) => MediaQuery.of(context).size.height;